import { Subject } from 'rxjs';
import { isBrowser, isWebSocketSupported, log } from './misc/util';
import { Message, signaling as proto } from './proto';
var MAXIMUM_MISSED_HEARTBEAT = 3;
var HEARTBEAT_INTERVAL = 5000;
var heartbeatMsg = proto.Message.encode(proto.Message.create({ heartbeat: true })).finish();
export var SignalingState;
(function (SignalingState) {
    SignalingState[SignalingState["CONNECTING"] = 0] = "CONNECTING";
    SignalingState[SignalingState["OPEN"] = 1] = "OPEN";
    SignalingState[SignalingState["CHECKING"] = 2] = "CHECKING";
    SignalingState[SignalingState["CHECKED"] = 4] = "CHECKED";
    SignalingState[SignalingState["CLOSED"] = 3] = "CLOSED";
})(SignalingState || (SignalingState = {}));
var Signaling = (function () {
    function Signaling(wc, url) {
        this.STREAM_ID = 1;
        this.wc = wc;
        this.url = url;
        this.state = SignalingState.CLOSED;
        this.connected = false;
        this.missedHeartbeat = 0;
        this.streamSubject = new Subject();
        this.stateSubject = new Subject();
    }
    Object.defineProperty(Signaling.prototype, "messageFromStream", {
        get: function () {
            return this.streamSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Signaling.prototype.sendOverStream = function (msg) {
        log.signaling(this.wc.myId + ' Forward message', msg);
        this.send({
            content: {
                id: msg.recipientId,
                lastData: msg.content === undefined,
                data: Message.encode(Message.create(msg)).finish(),
            },
        });
    };
    Object.defineProperty(Signaling.prototype, "onState", {
        get: function () {
            return this.stateSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Signaling.prototype.check = function () {
        var _this = this;
        this.setState(SignalingState.CHECKING);
        this.send({
            connect: { id: this.wc.myId, members: this.wc.members.filter(function (id) { return id !== _this.wc.myId; }) },
        });
    };
    Signaling.prototype.connect = function (key) {
        var _this = this;
        if (isWebSocketSupported()) {
            this.setState(SignalingState.CONNECTING);
            this.ws = new global.WebSocket(this.fullUrl(key));
            this.ws.binaryType = 'arraybuffer';
            this.connectionTimeout = setTimeout(function () {
                if (_this.ws && _this.ws.readyState !== _this.ws.OPEN) {
                    log.signaling("Failed to connect to Signaling server " + _this.url + ": connection timeout");
                    _this.close();
                }
            }, 10000);
            this.ws.onopen = function () {
                _this.setState(SignalingState.OPEN);
                if (_this.connectionTimeout) {
                    clearTimeout(_this.connectionTimeout);
                }
                _this.startHeartbeat();
            };
            this.ws.onerror = function (err) { return log.signaling("WebSocket ERROR", err); };
            this.ws.onclose = function (closeEvt) {
                _this.clean();
                _this.setState(SignalingState.CLOSED);
            };
            this.ws.onmessage = function (_a) {
                var data = _a.data;
                return _this.handleMessage(data);
            };
        }
        else {
            throw new Error('Failed to join over Signaling: WebSocket is not supported by your environment');
        }
    };
    Signaling.prototype.close = function () {
        if (this.state !== SignalingState.CLOSED) {
            if (this.ws) {
                this.ws.onmessage = function () { };
                this.ws.onclose = function () { };
                this.ws.onerror = function () { };
                this.ws.close(1000);
            }
            this.clean();
            this.setState(SignalingState.CLOSED);
        }
    };
    Signaling.prototype.clean = function () {
        if (this.connectionTimeout) {
            global.clearTimeout(this.connectionTimeout);
        }
        if (this.heartbeatInterval) {
            global.clearInterval(this.heartbeatInterval);
        }
        this.ws = undefined;
    };
    Signaling.prototype.handleMessage = function (bytes) {
        var _this = this;
        var msg = proto.Message.decode(new Uint8Array(bytes));
        switch (msg.type) {
            case 'heartbeat':
                this.missedHeartbeat = 0;
                break;
            case 'connected':
                this.connected = msg.connected;
                this.setState(SignalingState.CHECKED);
                if (!msg.connected) {
                    this.wc.channelBuilder.connectOverSignaling().catch(function () { return _this.check(); });
                }
                break;
            case 'content': {
                var _a = msg.content, data = _a.data, id = _a.id;
                var streamMessage = Message.decode(data);
                streamMessage.senderId = id;
                log.signaling('StreamMessage RECEIVED: ', streamMessage);
                this.streamSubject.next(streamMessage);
                break;
            }
        }
    };
    Signaling.prototype.setState = function (state) {
        if (this.state !== state) {
            this.state = state;
            this.stateSubject.next(state);
        }
    };
    Signaling.prototype.startHeartbeat = function () {
        var _this = this;
        this.missedHeartbeat = 0;
        this.heartbeatInterval = global.setInterval(function () {
            try {
                _this.missedHeartbeat++;
                if (_this.missedHeartbeat >= MAXIMUM_MISSED_HEARTBEAT) {
                    throw new Error('Too many missed heartbeats');
                }
                _this.heartbeat();
            }
            catch (err) {
                _this.close();
            }
        }, HEARTBEAT_INTERVAL);
    };
    Signaling.prototype.send = function (msg) {
        if (this.ws && this.ws.readyState === WebSocket.OPEN) {
            try {
                this.ws.send(proto.Message.encode(proto.Message.create(msg)).finish());
            }
            catch (err) {
                log.signaling('Failed send to Signaling', err);
            }
        }
    };
    Signaling.prototype.heartbeat = function () {
        if (this.ws && this.ws.readyState === WebSocket.OPEN) {
            try {
                this.ws.send(heartbeatMsg);
            }
            catch (err) {
                log.signaling('Failed send to Signaling: ' + err.message);
            }
        }
    };
    Signaling.prototype.fullUrl = function (key) {
        var urlWithKey = this.url.endsWith('/') ? this.url + key : this.url + '/' + key;
        if (isBrowser) {
            return urlWithKey;
        }
        else {
            return urlWithKey + '?favored';
        }
    };
    return Signaling;
}());
export { Signaling };
//# sourceMappingURL=Signaling.js.map